#!/usr/bin/env python
# Icinga 2
# Copyright (C) 2012-2017 Icinga Development Team (https://www.icinga.com/)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import subprocess, json

inventory_json = subprocess.check_output(["icinga2", "agent", "list", "--batch"])
inventory = json.loads(inventory_json)

for agent, agent_info in inventory.items():
    print "object Endpoint \"%s\" {" % (agent)
    print "  host = \"%s\"" % (agent)
    print "}"
    print ""
    print "object Zone \"%s\" {" % (agent_info["zone"])
    if "parent_zone" in agent_info:
        print "  parent = \"%s\"" % (agent_info["parent_zone"])
    print "  endpoints = [ \"%s\" ]" % (agent)
    print "}"
    print ""

    print "object Host \"%s\" {" % (agent_info["zone"])
    print "  check_command = \"cluster-zone\""
    print "}"
    print ""

    print "apply Dependency \"host-zone-%s\" to Host {" % (agent_info["zone"])
    print "  parent_host_name = \"%s\"" % (agent_info["zone"])
    print "  assign where host.zone == \"%s\"" % (agent_info["zone"])
    print "}"
    print ""

    print "apply Dependency \"service-zone-%s\" to Service {" % (agent_info["zone"])
    print "  parent_host_name = \"%s\"" % (agent_info["zone"])
    print "  assign where service.zone == \"%s\"" % (agent_info["zone"])
    print "}"
    print ""

    for host, services in agent_info["repository"].items():
        if host != agent_info["zone"]:
            print "object Host \"%s\" {" % (host)
            print "  check_command = \"dummy\""
            print "  zone = \"%s\"" % (agent_info["zone"])
            print "}"
            print ""

        for service in services:
            print "object Service \"%s\" {" % (service)
            print "  check_command = \"dummy\""
            print "  host_name = \"%s\"" % (host)
            print "  zone = \"%s\"" % (agent_info["zone"])
            print "}"
            print ""




#!/usr/bin/env python
# Icinga 2
# Copyright (C) 2012-2017 Icinga Development Team (https://www.icinga.com/)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import sys
import subprocess
import socket
import urlparse
import requests
import json
from xml.dom.minidom import parse

api_url = "https://localhost:5665/"
api_user = "root"
api_password = "root"

if len(sys.argv) < 2:
    print "Syntax: %s <xml-file> [<xml-file> ...]" % (sys.argv[0])
    sys.exit(1)

tcp_service_commands = {
  'ssh': 'ssh',
  'http': 'http',
  'smtp': 'smtp',
  'ssmtp': 'ssmtp'
}

udp_service_commands = {
  'ntp': 'ntp_time',
  'snmp': 'snmp-uptime'
}

hosts = {}

def process_host(host_element):
    global hosts

    status = "down"

    for status_element in host_element.getElementsByTagName("status"):
        status = status_element.getAttribute("state")

    if status != "up":
        return

    for address_element in host_element.getElementsByTagName("address"):
        if not address_element.getAttribute("addrtype") in [ "ipv4", "ipv6" ]:
            continue

        address = address_element.getAttribute("addr")
        break

    name = address

    for hostname_element in host_element.getElementsByTagName("hostname"):
        name = hostname_element.getAttribute("name")

    try:
        services = hosts[name]["services"]
    except:
        services = {}

    for port_element in host_element.getElementsByTagName("port"):
        state = "closed"

        for state_element in port_element.getElementsByTagName("state"):
            state = state_element.getAttribute("state")

        if state != "open":
            continue

        port = int(port_element.getAttribute("portid"))
        protocol = port_element.getAttribute("protocol")

        try:
            serv = socket.getservbyport(port, protocol)
        except:
            serv = str(port)

        try:
            if protocol == "tcp":
                command = tcp_service_commands[serv]
            elif protocol == "udp":
                command = udp_service_commands[serv]
            else:
                raise "Unknown protocol."
        except:
            command = protocol

        if command == "udp":
            continue

        services[serv] = { "command": command, "port": port }

    hosts[name] = { "name": name, "address": address, "services": services }

def create_host(host):
    global api_url, api_user, api_password

    req = {
      "templates": [ "discovered-host" ],
      "attrs": {
        "address": host["address"]
      }
    }

    headers = {"Accept": "application/json"}
    url = urlparse.urljoin(api_url, "v1/objects/hosts/%s" % (host["name"]))
    requests.put(url, headers=headers, auth=(api_user, api_password), data=json.dumps(req), verify=False)

    for serv, service in host["services"].iteritems():
        req = {
          "templates": [ "discovered-service" ],
          "attrs": {
            "vars.%s_port" % (service["command"]): service["port"],
            "check_command": service["command"],
          }
        }
    
        headers = {"Accept": "application/json"}
        url = urlparse.urljoin(api_url, "v1/objects/services/%s!%s" % (host["name"], serv))
        requests.put(url, headers=headers, auth=(api_user, api_password), data=json.dumps(req), verify=False)

for arg in sys.argv[1:]:
    # Expects XML output from 'nmap -oX'
    dom = parse(arg)

    for host in dom.getElementsByTagName("host"):
        process_host(host)

for host in hosts.values():
    create_host(host)



from __future__ import unicode_literals

import sys
from datetime import datetime, timedelta

import utils

CHECK_INTERVAL = 10 # minutes; The actual interval is 5 minutes but as other
                    # tests might restart Icinga we need to take any
                    # rescheduling into account

TABLE_PREFIX = 'icinga_'
TABLES = [
    # Central tables
    'instances',
    'objects',
    # Debugging tables
    'conninfo',
    # Historical tables
    'acknowledgements',
    'commenthistory',
    'contactnotifications',
    'dbversion',
    'downtimehistory',
    'eventhandlers',
    'externalcommands',
    'flappinghistory',
    'hostchecks',
    'logentries',
    'notifications',
    'processevents',
    'servicechecks',
    'statehistory',
    'systemcommands',
    # Current status tables
    'comments',
    'customvariablestatus',
    'hoststatus',
    'programstatus',
    'runtimevariables',
    'scheduleddowntime',
    'servicestatus',
    'contactstatus',
    # Configuration tables
    'commands',
    'configfiles',
    'configfilevariables',
    'contact_addresses',
    'contact_notificationcommands',
    'contactgroup_members',
    'contactgroups',
    'contactnotificationmethods',
    'contacts',
    'customvariables',
    'host_contactgroups',
    'host_contacts',
    'host_parenthosts',
    'hostdependencies',
    'hostescalation_contactgroups',
    'hostescalation_contacts',
    'hostescalations',
    'hostgroup_members',
    'hostgroups',
    'hosts',
    'service_contactgroups',
    'service_contacts',
    'servicedependencies',
    'serviceescalation_contactgroups',
    'serviceescalation_contacts',
    'serviceescalations',
    'servicegroup_members',
    'servicegroups',
    'services',
    'timeperiod_timeranges',
    'timeperiods'
    ]
EXAMPLE_CONFIG = {
    'localhost': ['disk', 'http', 'icinga', 'load', 'ping4',
                  'ping6', 'procs', 'ssh', 'users'],
    'nsca-ng': ['PassiveService1', 'PassiveService2']
}


def validate_tables(tables):
    """
    Return whether all tables of the IDO database scheme exist in
    the given table listing

    """
    utils.Logger.info('Checking database scheme... (tables)\n')
    failures = False
    for table in (TABLE_PREFIX + n for n in TABLES):
        if table in tables:
            utils.Logger.ok('Found table "{0}" in database\n'.format(table))
        else:
            utils.Logger.fail('Could not find table "{0}" in database\n'
                              ''.format(table))
            failures = True

    return not failures


def verify_host_config(config_data):
    """
    Return whether the example hosts exist in the given "hosts" table

    """
    utils.Logger.info('Checking example host configuration...\n')
    failures = False
    for hostname in EXAMPLE_CONFIG:
        if not any(1 for e in config_data if e['alias'] == hostname):
            utils.Logger.fail('Could not find host "{0}"\n'.format(hostname))
            failures = True
        else:
            utils.Logger.ok('Found host "{0}"\n'.format(hostname))

    return not failures


def verify_service_config(config_data):
    """
    Return whether the example services exist in the given "services" table

    """
    utils.Logger.info('Checking example service configuration...\n')
    failures = False
    for hostname, servicename in ((h, s) for h, ss in EXAMPLE_CONFIG.iteritems()
                                         for s in ss):
        if not any(1 for c in config_data
                     if c['alias'] == hostname and
                        c['display_name'] == servicename):
            utils.Logger.fail('Could not find service "{0}" on host "{1}"\n'
                              ''.format(servicename, hostname))
            failures = True
        else:
            utils.Logger.ok('Found service "{0}" on host "{1}"\n'
                            ''.format(servicename, hostname))

    return not failures


def check_last_host_status_update(check_info):
    """
    Return whether the example hosts are checked as scheduled

    """
    utils.Logger.info('Checking last host status updates...\n')
    failures = False
    for host_info in check_info:
        if host_info['alias'] == 'localhost':
            last_check = datetime.fromtimestamp(float(host_info['last_check']))
            if datetime.now() - last_check > timedelta(minutes=CHECK_INTERVAL,
                                                       seconds=10):
                utils.Logger.fail('The last status update of host "{0}" was'
                                  ' more than {1} minutes ago\n'
                                  ''.format(host_info['alias'], CHECK_INTERVAL))
                failures = True
            else:
                utils.Logger.ok('Host "{0}" is being updated\n'
                                ''.format(host_info['alias']))
        elif host_info['alias'] == 'nsca-ng':
            if float(host_info['last_check']) > 0:
                utils.Logger.fail('The host "{0}" was checked even'
                                  ' though it has no check service'
                                  ''.format(host_info['alias']))
                failures = True
            else:
                utils.Logger.ok('Host "{0}" is not being checked because '
                                'there is no check service\n'
                                ''.format(host_info['alias']))
        else:
            utils.Logger.info('Skipping host "{0}"\n'
                              ''.format(host_info['alias']))

    return not failures


def check_last_service_status_update(check_info):
    """
    Return whether the example services are checked as scheduled

    """
    utils.Logger.info('Checking last service status updates...\n')
    failures = False
    for svc_info in check_info:
        if svc_info['display_name'] in EXAMPLE_CONFIG.get(svc_info['alias'], []):
            last_check = datetime.fromtimestamp(float(svc_info['last_check']))
            if datetime.now() - last_check > timedelta(minutes=CHECK_INTERVAL,
                                                       seconds=10):
                utils.Logger.fail('The last status update of service "{0}" on '
                                  'host "{1}" was more than {2} minutes ago\n'
                                  ''.format(svc_info['display_name'],
                                            svc_info['alias'],
                                            CHECK_INTERVAL))
                failures = True
            else:
                utils.Logger.ok('Service "{0}" on host "{1}" is being updated\n'
                                ''.format(svc_info['display_name'],
                                          svc_info['alias']))
        else:
            utils.Logger.info('Skipping service "{0}" on host "{1}"\n'
                              ''.format(svc_info['display_name'],
                                        svc_info['alias']))

    return not failures


def check_logentries(logentry_info):
    """
    Return whether the given logentry originates from host "localhost"
    and refers to its very last hard status change

    """
    utils.Logger.info('Checking status log for host "localhost"...\n')
    if logentry_info and logentry_info[0]['alias'] == 'localhost':
        entry_time = datetime.fromtimestamp(float(logentry_info[0]['entry_time']))
        state_time = datetime.fromtimestamp(float(logentry_info[0]['state_time']))
        if entry_time - state_time > timedelta(seconds=10):
            utils.Logger.fail('The last hard state of host "localhost"'
                              ' seems not to have been logged\n')
            return False
    else:
        utils.Logger.fail('No logs found in the IDO for host "localhost"\n')
        return False

    utils.Logger.ok('The last hard state of host "localhost"'
                    ' was properly logged\n')
    return True




from __future__ import unicode_literals

import os
import sys
import time
import json
import socket
import subprocess

__all__ = ['parse_statusdata', 'run_mysql_query', 'run_pgsql_query',
           'LiveStatusSocket']


MYSQL_PARAMS = b"-t -D icinga -u icinga --password=icinga -e".split()
MYSQL_SEPARATOR = '|'

PGSQL_PARAMS = b"-nq -U icinga -d icinga -c".split()
PGSQL_SEPARATOR = '|'
PGSQL_ENVIRONMENT = {
    b'PGPASSWORD': b'icinga'
    }


def parse_statusdata(data, intelligent_cast=True):
    parsed_data, data_type, type_data = {}, '', {}
    for line in (l for l in data.split(os.linesep)
                   if l and not l.startswith('#')):
        if '{' in line:
            data_type = line.partition('{')[0].strip()
        elif '}' in line:
            parsed_data.setdefault(data_type, []).append(type_data)
        else:
            key, _, value = line.partition('=')

            if intelligent_cast:
                value = _cast_status_value(value)

            type_data[key.strip()] = value

    return parsed_data


def _cast_status_value(value):
    try:
        return int(value)
    except ValueError:
        try:
            return float(value)
        except ValueError:
            return value


def run_mysql_query(query, path):
    p = subprocess.Popen([path] + MYSQL_PARAMS + [query.encode('utf-8')],
                         stdout=subprocess.PIPE)
    Logger.debug('Sent MYSQL query: {0!r}\n'.format(query))
    resultset = [l.decode('utf-8') for l in p.stdout.readlines()]
    Logger.debug('Received MYSQL resultset: {0!r}\n'
                 ''.format(''.join(resultset)), True)
    return _parse_mysql_result(resultset)


def _parse_mysql_result(resultset):
    result, header = [], None
    for line in (l for l in resultset if MYSQL_SEPARATOR in l):
        columns = [c.strip() for c in line[1:-3].split(MYSQL_SEPARATOR)]
        if header is None:
            header = columns
        else:
            result.append(dict((header[i], v if v != 'NULL' else None)
                               for i, v in enumerate(columns)))
    return result


def run_pgsql_query(query, path):
    p = subprocess.Popen([path] + PGSQL_PARAMS + [query.encode('utf-8')],
                         stdout=subprocess.PIPE, env=PGSQL_ENVIRONMENT)
    Logger.debug('Sent PostgreSQL query: {0!r}\n'.format(query))
    resultset = [l.decode('utf-8') for l in p.stdout.readlines()]
    Logger.debug('Received PostgreSQL resultset: {0!r}\n'
                 ''.format(''.join(resultset)), True)
    return _parse_pgsql_result(resultset)


def _parse_pgsql_result(resultset):
    result, header = [], None
    for line in (l for l in resultset if PGSQL_SEPARATOR in l):
        columns = [c.strip() for c in line.split(PGSQL_SEPARATOR)]
        if header is None:
            header = columns
        else:
            result.append(dict((header[i], v) for i, v in enumerate(columns)))
    return result


class LiveStatusError(Exception):
    pass


class LiveStatusSocket(object):
    options = [
        'KeepAlive: on',
        'OutputFormat: json',
        'ResponseHeader: fixed16'
        ]

    def __init__(self, path):
        self.path = path

        self._connected = False

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.close()

    def connect(self):
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        Logger.debug('Opened UNIX stream socket\n', True)
        self.sock.connect(self.path)
        Logger.debug('Connected to Livestatus socket: {0}\n'.format(self.path),
                     True)
        self._connected = True

    def reconnect(self, timeout=30):
        Logger.debug('Reconnecting to Livestatus socket\n', True)
        start = time.time()
        while not self._connected and time.time() - start < timeout:
            try:
                self.connect()
            except socket.error, error:
                Logger.debug('Could not connect: {0}\n'.format(error), True)
                # Icinga2 does some "magic" with the socket during startup
                # which causes random errors being raised (EACCES, ENOENT, ..)
                # so we just ignore them until the timeout is reached
                time.sleep(1)
        if not self._connected:
            # Raise the very last exception once the timeout is reached
            raise

    def close(self):
        if self._connected:
            self.sock.shutdown(socket.SHUT_RDWR)
            Logger.debug('Shutted down Livestatus connection\n', True)
            self.sock.close()
            Logger.debug('Closed Livestatus socket\n', True)
            self._connected = False

    def query(self, command):
        self.send(command)
        statuscode, response = self.recv()

        if statuscode != 200:
            raise LiveStatusError(statuscode, response)

        return response

    def send(self, query):
        if not self._connected:
            raise RuntimeError('Tried to write to closed socket')

        full_query = '\n'.join([query] + self.options)
        self.sock.sendall((full_query + '\n\n').encode('utf-8'))
        Logger.debug('Sent Livestatus query: {0!r}\n'.format(full_query))

    def recv(self):
        if not self._connected:
            raise RuntimeError('Tried to read from closed socket')

        response = b''
        response_header = self.sock.recv(16)
        response_code = int(response_header[:3])
        response_length = int(response_header[3:].strip())

        if response_length > 0:
            while len(response) < response_length:
                response += self.sock.recv(response_length - len(response))

            response = response.decode('utf-8')

            try:
                response = json.loads(response)
            except ValueError:
                pass

        Logger.debug('Received Livestatus response: {0!r} (Header was: {1!r})'
                     '\n'.format(response, response_header), True)
        return response_code, response


class Logger(object):
    INFO = 1
    OK = 2
    FAIL = 3
    ERROR = 4
    DEBUG_STD = 5
    DEBUG_EXT = 6

    VERBOSITY = None

    @classmethod
    def permitted(cls, severity):
        if cls.VERBOSITY is None:
            cls.VERBOSITY = next((int(sys.argv.pop(i).partition('=')[2])
                                  for i, a in enumerate(sys.argv)
                                  if a.startswith('--verbosity=')), 1)

        return (severity == cls.INFO and cls.VERBOSITY >= 1) or \
               (severity == cls.OK and cls.VERBOSITY >= 1) or \
               (severity == cls.FAIL and cls.VERBOSITY >= 1) or \
               (severity == cls.ERROR and cls.VERBOSITY >= 1) or \
               (severity == cls.DEBUG_STD and cls.VERBOSITY >= 2) or \
               (severity == cls.DEBUG_EXT and cls.VERBOSITY >= 3)

    @staticmethod
    def write(text, stderr=False):
        if stderr:
            sys.stderr.write(text)
            sys.stderr.flush()
        else:
            sys.stdout.write(text)
            sys.stdout.flush()

    @classmethod
    def log(cls, severity, text):
        if severity == cls.INFO and cls.permitted(cls.INFO):
            cls.write('\x00[INFO] {0}'.format(text))
        elif severity == cls.ERROR and cls.permitted(cls.ERROR):
            cls.write('\x00[ERROR] {0}'.format(text))
        elif severity == cls.FAIL and cls.permitted(cls.FAIL):
            cls.write('\x00[FAIL] {0}'.format(text))
        elif severity == cls.OK and cls.permitted(cls.OK):
            cls.write('\x00[OK] {0}'.format(text))
        elif severity == cls.DEBUG_STD and cls.permitted(cls.DEBUG_STD):
            cls.write('\x00\x00[DEBUG] {0}'.format(text))
        elif severity == cls.DEBUG_EXT and cls.permitted(cls.DEBUG_EXT):
            cls.write('\x00\x00\x00\x00[DEBUG] {0}'.format(text))
        else:
            return False
        return True

    @classmethod
    def info(cls, text):
        return cls.log(cls.INFO, text)

    @classmethod
    def error(cls, text):
        return cls.log(cls.ERROR, text)

    @classmethod
    def fail(cls, text):
        return cls.log(cls.FAIL, text)

    @classmethod
    def ok(cls, text):
        return cls.log(cls.OK, text)

    @classmethod
    def debug(cls, text, extended=False):
        return cls.log(cls.DEBUG_EXT if extended else cls.DEBUG_STD, text)




#!/usr/bin/env python
from __future__ import unicode_literals


import os
import re
import sys
import json
import glob
import subprocess
from optparse import OptionParser
from xml.dom.minidom import getDOMImplementation


try:
    from subprocess import DEVNULL
except ImportError:
    DEVNULL = open(os.devnull, 'w')


class Logger(object):
    INFO = 1
    OK = 2
    FAIL = 3
    ERROR = 4
    DEBUG_STD = 5
    DEBUG_EXT = 6

    VERBOSITY = 0
    OUTPUT_LENGTH = 1024

    @staticmethod
    def write(text, stderr=False):
        if stderr:
            sys.stderr.write(text.encode('utf-8'))
            sys.stderr.flush()
        else:
            sys.stdout.write(text.encode('utf-8'))
            sys.stdout.flush()

    @classmethod
    def set_verbosity(cls, verbosity):
        cls.VERBOSITY = verbosity

    @classmethod
    def log(cls, severity, text):
        if severity == cls.INFO and cls.VERBOSITY >= 1:
            cls.write('\033[1;94m[INFO]\033[1;0m {0}'.format(text))
        elif severity == cls.ERROR and cls.VERBOSITY >= 1:
            cls.write('\033[1;33m[ERROR]\033[1;0m {0}'.format(text), True)
        elif severity == cls.FAIL and cls.VERBOSITY >= 1:
            cls.write('\033[1;31m[FAIL] {0}\033[1;0m'.format(text))
        elif severity == cls.OK and cls.VERBOSITY >= 1:
            cls.write('\033[1;32m[OK]\033[1;0m {0}'.format(text))
        elif severity == cls.DEBUG_STD and cls.VERBOSITY >= 2:
            cls.write('\033[1;90m[DEBUG]\033[1;0m {0}'.format(text))
        elif severity == cls.DEBUG_EXT and cls.VERBOSITY >= 3:
            if cls.VERBOSITY < 4 and len(text) > cls.OUTPUT_LENGTH:
                suffix = '... (Truncated to {0} bytes)\n' \
                         ''.format(cls.OUTPUT_LENGTH)
                text = text[:cls.OUTPUT_LENGTH] + suffix
            cls.write('\033[1;90m[DEBUG]\033[1;0m {0}'.format(text))
        else:
            return False
        return True

    @classmethod
    def info(cls, text):
        return cls.log(cls.INFO, text)

    @classmethod
    def error(cls, text):
        return cls.log(cls.ERROR, text)

    @classmethod
    def fail(cls, text):
        return cls.log(cls.FAIL, text)

    @classmethod
    def ok(cls, text):
        return cls.log(cls.OK, text)

    @classmethod
    def debug(cls, text, extended=False):
        return cls.log(cls.DEBUG_EXT if extended else cls.DEBUG_STD, text)


class TestSuite(object):
    def __init__(self, configpath):
        self._tests = []
        self._results = {}

        self.load_config(configpath)

    def add_test(self, filepath):
        self._tests.append(filepath)

    def load_config(self, filepath):
        with open(filepath) as f:
            self._config = json.load(f)

    def get_report(self):
        dom = getDOMImplementation()
        document = dom.createDocument(None, 'testsuite', None)
        xml_root = document.documentElement

        for name, info in self._results.iteritems():
            testresult = document.createElement('testcase')
            testresult.setAttribute('classname', 'vm')
            testresult.setAttribute('name', name)

            totaltests = document.createElement('tests')
            totaltests.appendChild(document.createTextNode(str(info['total'])))
            testresult.appendChild(totaltests)

            failedtests = document.createElement('failures')
            failedtests.appendChild(document.createTextNode(str(info['failures'])))
            testresult.appendChild(failedtests)

            systemout = document.createElement('system-out')
            systemout.appendChild(document.createTextNode(info['stdout']))
            testresult.appendChild(systemout)

            systemerr = document.createElement('system-err')
            systemerr.appendChild(document.createTextNode(info['stderr']))
            testresult.appendChild(systemerr)

            if info['returncode'] != 0:
                failure = document.createElement('failure')
                failure.setAttribute('type', 'returncode')
                failure.appendChild(document.createTextNode(
                    'code: {0}'.format(info['returncode'])))
                testresult.appendChild(failure)

            xml_root.appendChild(testresult)

        return document.toxml()

    def run(self):
        for path in self._tests:
            test_name = os.path.basename(path)
            Logger.debug('Copying test "{0}" to remote machine\n'.format(test_name))
            self._copy_test(path)
            self._apply_setup_routines(test_name, 'setup')
            note_printed = Logger.info('Running test "{0}"...\n'.format(test_name))
            result = self._run_test(path)
            Logger.info('Test "{0}" has finished (Total tests: {1}, Failures: {2})\n'
                        ''.format(test_name, result['total'], result['failures']))
            self._apply_setup_routines(test_name, 'teardown')
            Logger.debug('Removing test "{0}" from remote machine\n'.format(test_name))
            self._remove_test(test_name)
            self._results[test_name] = result
            if note_printed:
                Logger.write('\n')

    def _apply_setup_routines(self, test_name, context):
        instructions = next((t[1].get(context)
                             for t in self._config.get('setups', {}).iteritems()
                             if re.match(t[0], test_name)), None)
        if instructions is not None:
            note_printed = Logger.info('Applying {0} routines for test "{1}" .. '
                                       ''.format(context, test_name))
            for instruction in instructions.get('copy', []):
                source, _, destination = instruction.partition('>>')
                self._copy_file(source.strip(), destination.strip())
            for filepath in instructions.get('clean', []):
                self._remove_file(filepath)
            for command in instructions.get('exec', []):
                self._exec_command(command)
            if note_printed:
                Logger.write('Done\n')

    def _remove_file(self, path):
        command = self._config['commands']['clean'].format(path)
        rc = subprocess.call(command, stdout=DEVNULL, shell=True)
        if rc != 0:
            Logger.error('Cannot remove file "{0}" ({1})\n'.format(path, rc))

    def _exec_command(self, command):
        command = self._config['commands']['exec'].format(command)
        rc = subprocess.call(command, stdout=DEVNULL, shell=True)
        if rc != 0:
            Logger.error('Command "{0}" exited with exit code "{1}"\n' \
                         ''.format(command, rc))

    def _copy_file(self, source, destination):
        command = self._config['commands']['copy'].format(source, destination)
        rc = subprocess.call(command, stdout=DEVNULL, shell=True)
        if rc != 0:
            Logger.error('Cannot copy file "{0}" to "{1}" ({2})\n' \
                         ''.format(source, destination, rc))

    def _copy_test(self, path):
        self._copy_file(path, os.path.join(self._config['settings']['test_root'],
                                           os.path.basename(path)))

    def _remove_test(self, test_name):
        test_root = self._config['settings']['test_root']
        self._remove_file(os.path.join(test_root, test_name))

    def _run_test(self, path):
        command = self._config['commands']['exec']
        target = os.path.join(self._config['settings']['test_root'],
                              os.path.basename(path))
        options = ['--verbosity={0}'.format(Logger.VERBOSITY)]
        p = subprocess.Popen(command.format(' '.join([target] + options)),
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                             shell=True)
        output, test_count, failed_tests = self._watch_output(p.stdout)
        return {
            'total': test_count,
            'failures': failed_tests,
            'stdout': output,
            'stderr': p.stderr.read().decode('utf-8'),
            'returncode': p.wait()
            }

    def _watch_output(self, pipe):
        output, total, failures = '', 0, 0
        while True:
            line = pipe.readline().decode('utf-8')
            if not line:
                break

            verbosity_level = line.count('\x00')
            line = line[verbosity_level:]
            if line.startswith('[ERROR] '):
                Logger.error(line[8:])
            elif line.startswith('[DEBUG] '):
                Logger.debug(line[8:], verbosity_level == 4)
            elif line.startswith('[FAIL] '):
                Logger.fail(line[7:])
                failures += 1
                total += 1
            elif line.startswith('[OK] '):
                Logger.ok(line[5:])
                total += 1
            else:
                Logger.info(line.replace('[INFO] ', ''))

            output += line
        return (output, total, failures)


def parse_commandline():
    parser = OptionParser(version='0.5')
    parser.add_option('-C', '--config', default="run_tests.conf",
                      help='The path to the config file to use [%default]')
    parser.add_option('-R', '--results',
                      help='The file where to store the test results')
    parser.add_option('-v', '--verbose', action='count', default=1,
                      help='Be more verbose (Maximum output: -vvv)')
    parser.add_option('-q', '--quiet', action='count', default=0,
                      help='Be less verbose')
    return parser.parse_args()


def main():
    options, arguments = parse_commandline()
    suite = TestSuite(options.config)

    for path in (p for a in arguments for p in glob.glob(a)):
        suite.add_test(path)

    Logger.set_verbosity(options.verbose - options.quiet)
    suite.run()

    if options.results is not None:
        with open(options.results, 'w') as f:
            f.write(suite.get_report().encode('utf-8'))

    return 0


if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/env python
# -*- coding:utf-8 -*-

import requests
import re
import pickle
import sys
import os
from datetime import datetime
from collections import defaultdict
from collections import OrderedDict

#################################
## Env Config

try:
    github_auth_username = os.environ['ICINGA_GITHUB_AUTH_USERNAME']
except KeyError:
    print "ERROR: Environment variable 'ICINGA_GITHUB_AUTH_USERNAME' is not set."
    sys.exit(1)

try:
    github_auth_token = os.environ['ICINGA_GITHUB_AUTH_TOKEN']
except:
    print "ERROR: Environment variable 'ICINGA_GITHUB_AUTH_TOKEN' is not set."
    sys.exit(1)

try:
    project_name = os.environ['ICINGA_GITHUB_PROJECT']
except:
    print "ERROR: Environment variable 'ICINGA_GITHUB_PROJECT' is not set."
    sys.exit(1)

#################################
## Config

changelog_file = "CHANGELOG.md" # TODO: config param
debug = 1

# Keep this in sync with GitHub labels.
ignored_labels = [
    "high-priority", "low-priority",
    "bug", "enhancement",
    "needs-feedback", "question", "duplicate", "invalid", "wontfix",
    "backported", "build-fix"
]

# Selectively show and collect specific categories
#
# (category, list of case sensitive matching labels)
# The order is important!
# Keep this in sync with GitHub labels.
categories = OrderedDict(
[
    ("Enhancement", ["enhancement"]),
    ("Bug", ["bug", "crash"]),
    ("ITL", ["ITL"]),
    ("Documentation", ["Documentation"]),
    ("Support", ["code-quality", "Tests", "Packages", "Installation"])
]
)

#################################
## Helpers

def write_changelog(line):
    clfp.write(line + "\n")

def log(level, msg):
    if level <= debug:
        print " " + msg

def fetch_github_resources(uri, params = {}):
    resources = []

    url = 'https://api.github.com/repos/' + project_name + uri + "?per_page=100" # 100 is the maximum

    while True:
        log(2, "Requesting URL: " + url)
        resp = requests.get(url, auth=(github_auth_username, github_auth_token), params=params)
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise e

        data = resp.json()

        if len(data) == 0:
            break

        resources.extend(data)

        # fetch the next page from headers, do not count pages
        # http://engineering.hackerearth.com/2014/08/21/python-requests-module/
        if "next" in resp.links:
            url = resp.links['next']['url']
            log(2, "Found next link for Github pagination: " + url)
        else:
            break # no link found, we are done
            log(2, "No more pages to fetch, stop.")

    return resources

def issue_type(issue):
    issue_labels = [label["name"] for label in issue["labels"]]

    # start with the least important first (e.g. "Support", "Documentation", "Bug", "Enhancement" as order)
    for category in reversed(categories):
        labels = categories[category]

        for label in labels:
            if label in issue_labels:
                return category

    return "Support"

def escape_markdown(text):
    #tmp = text.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
    tmp = text
    tmp.replace('\\', '\\\\')

    return re.sub("([<>*_()\[\]#])", r"\\\1", tmp)

def format_labels(issue):
    labels = filter(lambda label: label not in ignored_labels, [label["name"] for label in issue["labels"]])

    # Mark PRs as custom label
    if "pull_request" in issue:
        labels.append("PR")

    if len(labels):
        return " (" + ", ".join(labels) + ")"
    else:
        return ""

def format_title(title):
    # Fix encoding
    try:
        issue_title = str(title.encode('ascii', 'ignore').encode('utf-8'))
    except Error:
        log(1, "Error: Cannot convert " + title + " to UTF-8")

    # Remove dev.icinga.com tag
    issue_title = re.sub('\[dev\.icinga\.com #\d+\] ', '', issue_title)

    #log(1, "Issue title: " + issue_title + "Type: " + str(type(issue_title)))

    return escape_markdown(issue_title)

#################################
## MAIN

milestones = {}
issues = defaultdict(lambda: defaultdict(list))

log(1, "Fetching data from GitHub API for project " + project_name)

try:
    tickets = fetch_github_resources("/issues", { "state": "all" })
except requests.exceptions.HTTPError as e:
    log(1, "ERROR " + str(e.response.status_code) + ": " + e.response.text)

    sys.exit(1)

clfp = open(changelog_file, "w+")

with open('tickets.pickle', 'wb') as fp:
    pickle.dump(tickets, fp)

with open('tickets.pickle', 'rb') as fp:
    cached_issues = pickle.load(fp)

for issue in cached_issues: #fetch_github_resources("/issues", { "state": "all" }):
    milestone = issue["milestone"]

    if not milestone:
        continue

    ms_title = milestone["title"]

    if not re.match('^\d+\.\d+\.\d+$', ms_title):
        continue

    if ms_title.split(".")[0] != "2":
        continue

    milestones[ms_title] = milestone

    ms_tickets = issues[ms_title][issue_type(issue)]
    ms_tickets.append(issue)

# TODO: Generic header based on project_name
write_changelog("# Icinga 2.x CHANGELOG")
write_changelog("")

for milestone in sorted(milestones.values(), key=lambda ms: (ms["due_on"], ms["title"]), reverse=True):
    if milestone["state"] != "closed":
        continue

    if milestone["due_on"] == None:
        print "Milestone", milestone["title"], "does not have a due date."
        sys.exit(1)

    ms_due_on = datetime.strptime(milestone["due_on"], "%Y-%m-%dT%H:%M:%SZ")

    write_changelog("## %s (%s)" % (milestone["title"], ms_due_on.strftime("%Y-%m-%d")))
    write_changelog("")

    ms_description = milestone["description"]
    ms_description = re.sub('\r\n', '\n', ms_description)

    if len(ms_description) > 0:
        write_changelog("### Notes\n\n" + ms_description + "\n") # Don't escape anything, we take care on Github for valid Markdown

    for category, labels in categories.iteritems():
        try:
            ms_issues = issues[milestone["title"]][category]
        except KeyError:
            continue

        if len(ms_issues) == 0:
            continue

        write_changelog("### " + category)
        write_changelog("")

        for issue in ms_issues:
            write_changelog("* [#" + str(issue["number"]) + "](https://github.com/" + project_name
                + "/issues/" + str(issue["number"]) + ")" + format_labels(issue) + ": " + format_title(issue["title"]))

        write_changelog("")

clfp.close()
log(1, "Finished writing " + changelog_file)



#!/usr/bin/env python
# Icinga 2
# Copyright (C) 2012-2017 Icinga Development Team (https://www.icinga.com/)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import os
import sys
import re

if len(sys.argv) < 2:
    print "Syntax: %s <md-files>" % sys.argv[0]
    print ""
    print "Updates inter-chapter links in the specified Markdown files."
    sys.exit(1)

anchors = {}

for file in sys.argv[1:]:
    text = open(file).read()
    for match in re.finditer(r"<a id=\"(?P<id>.*?)\">", text):
        id = match.group("id")

        if id in anchors:
            print "Error: Anchor '%s' is used multiple times: in %s and %s" % (id, file, anchors[id])

        anchors[match.group("id")] = file

def update_anchor(match):
    id = match.group("id")

    try:
        file = os.path.basename(anchors[id])
    except KeyError:
        print "Error: Unmatched anchor: %s" % (id)
        file = ""

    return "[%s](%s#%s)" % (match.group("text"), file, id)

for file in sys.argv[1:]:
    text = open(file).read()
    print "> Processing file '%s'..." % (file)
    new_text = re.sub(r"\[(?P<text>.*?)\]\((?P<file>[0-9-a-z\.]+)?#(?P<id>[^#\)]+)\)", update_anchor, text)
    open(file, "w").write(new_text)



#!/usr/bin/env python
# Icinga 2
# Copyright (C) 2012-2017 Icinga Development Team (https://www.icinga.com/)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

import sys
import subprocess
import socket
from xml.dom.minidom import parse

if len(sys.argv) < 2:
    print "Syntax: %s <xml-file> [<xml-file> ...]" % (sys.argv[0])
    sys.exit(1)

tcp_service_commands = {
  'ssh': 'ssh',
  'http': 'http_ip',
  'https': 'https_ip',
  'smtp': 'smtp',
  'ssmtp': 'ssmtp'
}

udp_service_commands = {
  'ntp': 'ntp_time',
  'snmp': 'snmp-uptime'
}

hosts = {}

def process_host(host_element):
    global hosts

    status = "down"

    for status_element in host_element.getElementsByTagName("status"):
        status = status_element.getAttribute("state")

    if status != "up":
        return

    for address_element in host_element.getElementsByTagName("address"):
        if not address_element.getAttribute("addrtype") in [ "ipv4", "ipv6" ]:
            continue

        address = address_element.getAttribute("addr")
        break

    name = address

    for hostname_element in host_element.getElementsByTagName("hostname"):
        name = hostname_element.getAttribute("name")

    try:
        services = hosts[name]["services"]
    except:
        services = {}

    for port_element in host_element.getElementsByTagName("port"):
        state = "closed"

        for state_element in port_element.getElementsByTagName("state"):
            state = state_element.getAttribute("state")

        if state != "open":
            continue

        port = int(port_element.getAttribute("portid"))
        protocol = port_element.getAttribute("protocol")

        try:
            serv = socket.getservbyport(port, protocol)
        except:
            serv = str(port)

        try:
            if protocol == "tcp":
                command = tcp_service_commands[serv]
            elif protocol == "udp":
                command = udp_service_commands[serv]
            else:
                raise "Unknown protocol."
        except:
            command = protocol

        if command == "udp":
            continue

        services[serv] = { "command": command, "port": port }

    hosts[name] = { "name": name, "address": address, "services": services }

def print_host(host):
    print "object Host \"%s\" {" % (host["name"])
    print "\timport \"discovered-host\","
    print ""
    print "\taddress = \"%s\"," % (host["address"])
    print "}"
    print ""

    for serv, service in host["services"].iteritems():
        print "object Service \"%s\" {" % (serv)
        print "\timport \"discovered-service\","
        print ""
        print "\thost_name = \"%s\"" % (host["name"])
        print "\tcheck_command = \"%s\"," % (service["command"])
        print ""
        print "\tvars.port = %s" % (service["port"])
        print "}"
        print ""

for arg in sys.argv[1:]:
    # Expects XML output from 'nmap -oX'
    dom = parse(arg)

    for host in dom.getElementsByTagName("host"):
        process_host(host)

for host in hosts.values():
    print_host(host)
